"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplicationServer = void 0;
require("reflect-metadata");
const express_1 = __importDefault(require("express"));
const Container_1 = require("../../DI/Container");
const PromiseHandler_1 = require("../../utils/PromiseHandler");
const printRoutes_1 = require("../../utils/printRoutes");
const functionCheck_1 = require("../../utils/functionCheck");
const createMappedRouter_1 = require("../helpers/createMappedRouter");
/**
 * @param ApplicationOptions: {   appHandler?: Application; port?: number; verbose?: "no" | "minimal" | "detailed"; controllers: Function[]; }
 *
 * * Initialize application with controllers.
 * * A new instance of app is created by default, but a pre-configured instance of express application can be used.
 * * Using the verbose option logs controller and route details to the console.
 */
function ApplicationServer({ controllers = [], port = 5000, appHandler, verbose = "no" }) {
    return function (Constructor) {
        const app = appHandler || express_1.default();
        let appConfig = {};
        if (!functionCheck_1.isFunctionTypeOnly(controllers))
            throw new Error("Invalid controllers");
        appConfig.controllers = controllers;
        const promiseHandler = new PromiseHandler_1.PromiseHandler();
        const applicationInstance = Container_1.container.resolveInstance(Constructor);
        for (const propName of Object.getOwnPropertyNames(Object.getPrototypeOf(applicationInstance))) {
            if (typeof applicationInstance[propName] !== "function")
                continue;
            if (Reflect.getMetadata("startup-component", applicationInstance, propName)) {
                const componentResult = applicationInstance[propName](app);
                if (componentResult instanceof Promise)
                    promiseHandler.addNewPromise(componentResult);
            }
            else if (Reflect.getMetadata("error-handler", applicationInstance, propName) && !appConfig.errorHandler) {
                const errorHandler = [].concat(Reflect.getMetadata("isFactory", applicationInstance, propName)
                    ? applicationInstance[propName]()
                    : applicationInstance[propName]);
                if (!functionCheck_1.isFunctionTypeOnly(errorHandler))
                    throw new Error("Invalid type: expected function");
                appConfig = Object.assign(Object.assign({}, appConfig), { errorHandler: errorHandler[0] });
            }
            else if (Reflect.getMetadata("application-catch-all", applicationInstance, propName) && !appConfig.catchAll) {
                const catchAll = [].concat(Reflect.getMetadata("isFactory", applicationInstance, propName)
                    ? applicationInstance[propName]()
                    : applicationInstance[propName]);
                if (!functionCheck_1.isFunctionTypeOnly(catchAll))
                    throw new Error("Invalid type: expected functions");
                appConfig = Object.assign(Object.assign({}, appConfig), { catchAll: catchAll[0] });
            }
            else if (Reflect.getMetadata("after-startup-component", applicationInstance, propName) &&
                !appConfig.afterStartupComponent) {
                appConfig = Object.assign(Object.assign({}, appConfig), { afterStartupComponent: applicationInstance[propName] });
            }
        }
        promiseHandler.once("success", () => {
            const appControllers = appConfig.controllers;
            if (!appControllers)
                throw new Error("No controllers to initialize");
            let printableModel = [];
            for (const controller of appControllers) {
                const controllerInstance = Container_1.container.resolveInstance(controller);
                const { router, model } = createMappedRouter_1.createMappedRouter(controllerInstance);
                printableModel.push(model);
                const metadata = Reflect.getMetadata("controller:metadata", Object.getPrototypeOf(controllerInstance));
                app.use(metadata.basePath, router);
            }
            const catchAll = appConfig.catchAll;
            catchAll && app.use("*", catchAll);
            const errorHandler = appConfig.errorHandler;
            errorHandler && app.use(errorHandler);
            verbose === "detailed"
                ? printableModel && printRoutes_1.pathMap.displayDetailedPathMap(printableModel)
                : verbose === "minimal"
                    ? printRoutes_1.pathMap.displayPathMap(app)
                    : null;
            const server = app.listen(port, () => __awaiter(this, void 0, void 0, function* () {
                console.log(`Server listening on port ${port}`);
                if (appConfig.afterStartupComponent) {
                    const afterEffects = appConfig.afterStartupComponent(server);
                    if (afterEffects instanceof Promise)
                        yield afterEffects;
                }
            }));
        });
        promiseHandler.once("failure", error => {
            console.log("Server failed to start with the error", error);
            process.exit(1);
        });
        promiseHandler.promises.length > 0 ? promiseHandler.executePromises() : promiseHandler.emit("success");
    };
}
exports.ApplicationServer = ApplicationServer;
//# sourceMappingURL=ApplicationServer.js.map