"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMappedRouter = void 0;
const express_1 = require("express");
const validateDto_1 = require("../../utils/validateDto");
function createMappedRouter(controllerInstance) {
    const router = express_1.Router();
    let controllerInfo = {
        controllerName: controllerInstance.constructor.name,
        entryMiddleware: [],
        exitMiddleware: [],
        endPoints: [],
    };
    const controllerMetadata = Reflect.getMetadata("controller:metadata", Object.getPrototypeOf(controllerInstance));
    let errorHandler = null;
    if (controllerMetadata.entryHandlers && controllerMetadata.entryHandlers.length > 0) {
        router.use(controllerMetadata.entryHandlers);
        controllerInfo.entryMiddleware = [
            ...controllerInfo.entryMiddleware,
            ...controllerMetadata.entryHandlers.map(handler => handler.name),
        ];
    }
    for (const propName of Object.getOwnPropertyNames(Object.getPrototypeOf(controllerInstance))) {
        if (typeof controllerInstance[propName] !== "function" || propName === "constructor")
            continue;
        if (Reflect.getMetadata("error-handler", controllerInstance, propName)) {
            errorHandler = controllerInstance[propName].bind(controllerInstance);
            continue;
        }
        const metaData = Reflect.getMetadata("route", controllerInstance, propName);
        if (!metaData || !metaData.endPoint)
            continue;
        let thisEndpoint = {
            path: metaData.endPoint,
            handlerName: propName,
            method: "GET",
            entryMiddleware: [],
            exitMiddleware: [],
        };
        if (metaData.preRouteHandlers) {
            router.use(metaData.endPoint, metaData.preRouteHandlers);
            thisEndpoint.entryMiddleware = [...thisEndpoint.entryMiddleware, ...metaData.preRouteHandlers.map(handler => handler.name)];
        }
        let endPointHandler = null;
        if (Reflect.getMetadata("isFactory", controllerInstance, propName)) {
            endPointHandler = [].concat(controllerInstance[propName].apply(controllerInstance, []));
        }
        else {
            endPointHandler = function (req, res, next) {
                return __awaiter(this, void 0, void 0, function* () {
                    let methodArguments = [];
                    const argEntity = Reflect.getMetadata("method:param", controllerInstance, propName);
                    if (argEntity) {
                        const params = argEntity.argModel;
                        const paramDataTypes = Reflect.getMetadata("design:paramtypes", controllerInstance, propName);
                        for (let index = 0; index < params.length; index++) {
                            const param = params[index];
                            let paramValidationSuccess = true;
                            switch (param.type) {
                                case "context":
                                    methodArguments.push({
                                        req,
                                        res,
                                        next,
                                    });
                                    break;
                                case "req":
                                    methodArguments.push(req);
                                    break;
                                case "res":
                                    methodArguments.push(res);
                                    break;
                                case "next":
                                    methodArguments.push(next);
                                    break;
                                case "body":
                                    paramValidationSuccess = validateDto_1.validateDto(req.body, paramDataTypes[index]);
                                    methodArguments.push(param.key ? req.body[param.key] : req.body);
                                    break;
                                case "param":
                                    methodArguments.push(param.key ? req.params[param.key] : req.params);
                                    break;
                                case "query":
                                    methodArguments.push(param.key ? req.query[param.key] : req.query);
                                    break;
                            }
                            if (!paramValidationSuccess)
                                return res.status(400).json({ status: "error", error: "invalid dto" });
                        }
                    }
                    let methodResult;
                    try {
                        methodResult = controllerInstance[propName].apply(controllerInstance, methodArguments);
                        if (methodResult instanceof Promise)
                            yield methodResult;
                    }
                    catch (error) {
                        return next(error);
                    }
                    if (!res.headersSent)
                        switch (typeof methodResult) {
                            case "string":
                                return res.send(methodResult);
                            case "object":
                                return res.json(methodResult);
                            case "bigint":
                            case "boolean":
                            case "number":
                                return res.send(methodResult.toString());
                            case "undefined":
                                break;
                            default:
                                return res.end();
                        }
                });
            };
        }
        switch (metaData.method) {
            case "get":
                router.get(metaData.endPoint, endPointHandler);
                thisEndpoint.method = "GET";
                break;
            case "post":
                router.post(metaData.endPoint, endPointHandler);
                thisEndpoint.method = "POST";
                break;
            case "put":
                router.put(metaData.endPoint, endPointHandler);
                thisEndpoint.method = "PUT";
                break;
            case "delete":
                router.delete(metaData.endPoint, endPointHandler);
                thisEndpoint.method = "DELETE";
                break;
        }
        if (metaData.postRouteHandlers) {
            router.use(`${metaData.endPoint}`, metaData.postRouteHandlers);
            thisEndpoint.exitMiddleware = [...thisEndpoint.exitMiddleware, ...metaData.postRouteHandlers.map(handler => handler.name)];
        }
        controllerInfo.endPoints.push(thisEndpoint);
    }
    if (controllerMetadata.exitHandlers && controllerMetadata.exitHandlers.length > 0) {
        router.use(controllerMetadata.exitHandlers);
        controllerInfo.exitMiddleware = [
            ...controllerInfo.exitMiddleware,
            ...controllerMetadata.exitHandlers.map(handler => handler.name),
        ];
    }
    if (errorHandler)
        router.use(errorHandler);
    return {
        router,
        model: controllerInfo,
    };
}
exports.createMappedRouter = createMappedRouter;
//# sourceMappingURL=createMappedRouter.js.map